/****************************************************************************/
/*            IDE Low Level Support Layer for TUOX OS v1.1		    */
/*    Written by Petr Pomkla & Karel Miarka, Technical University Ostrava   */
/*               Copyright (c)1997-98 Petr Pomkla. Jan 21 1998              */
/*    								            */                                                /*  Notes: CHS Addressing    Yes              */
/*     LBA Addressing    Yes              				    */
/*     PIO Mode          Yes   					            */
/****************************************************************************/
#include "ide.h"

/* Compilation Control*/
/* #define SHOW_COPYRIGHT */
/* #define SHOW_DRIVE_INFO */
/* #define SHOW_PARTITION_DETAIL */
/* #define TEST_CODE_ALLOWED */

/****************************************************************************/
/*      Global Variables                */
/****************************************************************************/
int PartCnt,TuoxPartCnt,DrvInit=0;
int TuoxPart[100];/* Partitions supported by TUOX OS */
PartInfo AllPart[100];/* All Partitions */
TypeDriveParam AllDrives[4];/* Drive Parameters */

/****************************************************************************/
/*      Converts Partition ID no. to String           */
/****************************************************************************/
#ifdef SHOW_PARTITION_DETAIL
char *IDtoText(int OSID){
  switch(OSID){
    case 0x00:return "Empty";
    case 0x01:return "DOS 12-bit FAT";
    case 0x02:return "XENIX root";
    case 0x03:return "XENIX usr";
    case 0x04:return "DOS 16-bit <32M";
    case 0x05:return "Extended";
    case 0x06:return "DOS 16-bit >=32M";
    case 0x07:return "OS/2 HPFS";
    case 0x08:return "AIX";
    case 0x09:return "AIX Bootable";
    case 0x0a:return "OS/2 Boot manager";
    case 0x0b:return "FAT32";
    case TUOX_ID:return "TUOX native";
    case 0x40:return "VENIX 80286";
    case 0x51:return "NOVELL?";
    case 0x52:return "MICROPORT";
    case 0x64:return "NOVELL NETWARE";
    case 0x65:return "NOVELL NETWARE";
    case 0x75:return "PC/IX";
    case 0x80:return "Old MINIX";
    case 0x81:return "LINUX/MINIX";
    case 0x82:return "LINUX swap";
    case 0x83:return "LINUX native";
    case 0x93:return "AMOBEA";
    case 0x94:return "AMOBEA BBT";
    case 0xa5:return "BSD/386";
    case 0xb7:return "BSDI fs";
    case 0xb8:return "BSDI swap";
    case 0xc7:return "SYRINX";
    case 0xdb:return "CP/M";
    case 0xe1:return "DOS access";
    case 0xe3:return "DOS r/o";
    case 0xf2:return "DOS secondary";
    case 0xff:return "BBT";
  }
 return "Unknown";
}
#endif
/****************************************************************************/
/*      Check if Partition's Item is Valid            */
/****************************************************************************/
int Valid(unsigned char *p2Buff){
  int i;
    for(i=0;i<16;i++)
      if(p2Buff[i]) return 1;
  return 0;
}
/****************************************************************************/
/*      Calculate Track no. from SectorTrack          */
/****************************************************************************/
unsigned short CalcTrack(unsigned short SectorTrack){
  return(SectorTrack>>8)|((SectorTrack&0xc0)<<2);
}/* Returns Track # */
/****************************************************************************/
/*      Read From IDE Register              */
/****************************************************************************/
unsigned char ReadIDEReg(unsigned IDEreg,int Primary){
  return inportb(IDEreg-(Primary?0:0x80));
}
/****************************************************************************/
/*      Write To IDE Register             */
/****************************************************************************/
void WriteIDEReg(unsigned IDEreg,unsigned char val,int Primary){
  outportb(IDEreg-(Primary?0:0x80),val);
}
/****************************************************************************/
/*      Read Data From IDE (PIO Mode)           */
/****************************************************************************/
 void ReadIDEData(register unsigned short *p2Buff,int sectors,int Primary){
  register unsigned short *p2Last=&p2Buff[sectors*256];
  unsigned short Reg=DATA_REG-(Primary?0:0x80);
    while(p2Buff<p2Last) *(p2Buff++)=inport(Reg);
}
/****************************************************************************/
/*      Write Data To IDE (PIO Mode)            */
/****************************************************************************/
 void WriteIDEData(register unsigned short *p2Buff,int sectors,int Primary){
  register unsigned short *p2Last=&p2Buff[sectors*256];
  unsigned short Reg=DATA_REG-(Primary?0:0x80);
    while(p2Buff<p2Last) outport(Reg,*(p2Buff++));
}
/****************************************************************************/
/*         High Level Command Interpreter         */
/****************************************************************************/
int IDEDisk(int cmd,unsigned char drive,unsigned char head,
      unsigned short track,unsigned char sector,
      unsigned char nsects,void *p2Buff,
      TypeDriveParam *p2DriveParam){
 int Primary=!((p2DriveParam[drive-0x80].Hddx>>1)&0x1),
     Master=!(p2DriveParam[drive-0x80].Hddx&0x1);
 unsigned long LBAAddress;
 unsigned char val;
   if (!p2DriveParam[drive-0x80].DriveValid) return -1;
   if (p2DriveParam[drive-0x80].LBAEnabled){/*Setup Drive in LBA Mode */
     LBAAddress=track*(unsigned long)(p2DriveParam[drive-0x80].MaxHead+1)
    *(unsigned long)p2DriveParam[drive-0x80].MaxSect
    +head*(unsigned long)p2DriveParam[drive-0x80].MaxSect+
    sector-1UL;
     WriteIDEReg(DRIVE_HEAD_REG,0xE0|(Master?0:0x10)|((LBAAddress>>24)&0x0f),
     Primary);/* Setup Drive/Head Register */
     WriteIDEReg(CYLINDER_HI_REG,(LBAAddress>>16)&0xff,Primary);/* Set CHigh */
     WriteIDEReg(CYLINDER_LOW_REG,(LBAAddress>>8)&0xff,Primary);/* Set CLow */
     WriteIDEReg(SECTOR_NUMBER_REG,LBAAddress&0xff,Primary);/* Set Sect Reg */
   } else {/* Setup drive in CHS Mode *//* Setup Drive/Head Register */
     WriteIDEReg(DRIVE_HEAD_REG,0xA0|(Master?0:0x10)|(head&0xf),Primary);
     WriteIDEReg(CYLINDER_LOW_REG,track&0xff,Primary);/* Setup Cyl. Low Reg*/
     WriteIDEReg(CYLINDER_HI_REG,track>>8,Primary);/* Setup Cyl. High Reg */
     WriteIDEReg(SECTOR_NUMBER_REG,sector,Primary);/* Setup Sector Reg */
   }
     WriteIDEReg(SECTOR_COUNT_REG,nsects,Primary);/* Setup Sector Count Reg */     WriteIDEReg(SECTOR_COUNT_REG,nsects,Primary);/* Setup Sector Count Reg */     WriteIDEReg(SECTOR_COUNT_REG,nsects,Primary);/* Setup Sector Count Reg */
   switch(cmd){
     case READ_DISK:
       /* Issue command (Read from Drive) */
       WriteIDEReg(COMMAND_REG,READ_SECTORS,Primary);
       while (nsects--){
   do{/* Wait Until Ready */
     val=ReadIDEReg(STATUS_REG,Primary);
     if(val&ERROR) return -2;
   }while (!(val&DATA_REQUEST)||(val&BUSY));
   /* Read IDE Buffer */
   ReadIDEData((unsigned short*)p2Buff,0x1,Primary);
   (unsigned short*)p2Buff+=256;
       }
     break;
     case WRITE_DISK:
       /* Issue command (Write to Drive) */
       WriteIDEReg(COMMAND_REG,WRITE_SECTORS,Primary);
       while(nsects--){
   do{/* Wait Until Ready */
     val=ReadIDEReg(STATUS_REG,Primary);
     if(val&ERROR) return -2;
   }while (!(val&DATA_REQUEST)||(val&BUSY));
   /* Write IDE Buffer */
   WriteIDEData((unsigned short*)p2Buff,0x1,Primary);
   (unsigned short*)p2Buff+=256;
       }
   do{/* Finish Write Operation */
     val=ReadIDEReg(STATUS_REG,Primary);
     if(val&ERROR) return -2;
   }while (val&BUSY);
     break;
     default:return -3;
  }
  return 0;
}
/****************************************************************************/
/*         Identify specified drive           */
/****************************************************************************/
int IdentifyDrive(int Master,int Primary,int BiosDrive,
      TypeDriveParam *p2DriveParam){
  int i;
  unsigned char val;
  unsigned short p2Buff[256];
  union REGS InRegs,OutRegs;
    /* Get BIOS Geometry */
    InRegs.h.ah=0x8;
    InRegs.h.dl=BiosDrive;
    int86(0x13,&InRegs,&OutRegs);
    if(OutRegs.x.cflag) return -1;
    p2DriveParam->DriveValid=1;
    p2DriveParam->MaxHead=OutRegs.h.dh;
    p2DriveParam->MaxSect=OutRegs.x.cx&0x3f;
    p2DriveParam->MaxCyl=(OutRegs.x.cx>>8)|((OutRegs.x.cx<<2)&0x300);
    /* Detect LBA mode */
    p2DriveParam->LBAEnabled=LBA_MODE&ReadIDEReg(DRIVE_HEAD_REG,Primary);
    /* Select Drive */
    WriteIDEReg(DRIVE_HEAD_REG,0xA0|(Master?MASTER_DRIVE:SLAVE_DRIVE),
    Primary);
    /* Issue command (Identify Drive) */
    WriteIDEReg(COMMAND_REG,IDENTIFY_DRIVE,Primary);
    /* Wait until Ready */
    do{
      val=ReadIDEReg(STATUS_REG,Primary);
      if(val&ERROR) return -2;
    }while (!(val&DATA_REQUEST)||(val&BUSY));
    /* Read IDE Buffer */
    ReadIDEData(p2Buff,1,Primary);
    /* Set drive ID */
    p2DriveParam->Hddx=!Master;
    p2DriveParam->Hddx+=Primary?0:2;
    /* Copy Model ID */
    if(p2Buff[27]){memcpy(p2DriveParam->Model,&p2Buff[27],40);
      p2DriveParam->Model[40]=0;}
    else *(p2DriveParam->Model)=0;
    for(i=39;i>=0;i--)
      if(p2DriveParam->Model[i]!=' '){p2DriveParam->Model[i+1]=0x0;break;}
  return 0;/*0 - Success, -1 - Bios geometry Error, -2 - IDE Ident. Error */
}
/****************************************************************************/
/*         Identify all drives in your system           */
/****************************************************************************/
void IdentifyDrives(TypeDriveParam *p2DriveParameters){
  int BiosDrive,Master=1,Primary=1,Status;
    memset (p2DriveParameters,0,4*sizeof(TypeDriveParam));/* Init Info Block*/
    for (BiosDrive=0x80;BiosDrive<0x84;BiosDrive++){/* Check All Bios Drives*/
      do{
  Status=IdentifyDrive(Master,Primary,BiosDrive,/* Identify Drive */
           &p2DriveParameters[BiosDrive-0x80]);
  if(Status==-1) return;/* Unable to get drive geometry */
    if(!Master&&!Primary) return;/* Last Physical Drive */
      else {Master=!Master;if(Master) Primary=!Primary;}/* Next Drive*/
  if(!Status) break;/* Drive Identification success */
      }while(1);
    }
}
/****************************************************************************/
/*         Scans all drives in your system          */
/****************************************************************************/
int ScanDrives(int *CurrPart,PartInfo *AllPart,TypeDriveParam *p2DriveParam){
  unsigned char MBR[512],Temp[512];
  unsigned char FHead;
  int CurrDrive,CurrMBROff,i,ff;
  unsigned short FSectorTrack;/* Check all drives hda .. hdd */
    for (*CurrPart=0x0,CurrDrive=0x80;CurrDrive<0x84;CurrDrive++){
      /* Scan Drive 'CurrDrive' */
      MBR[0x1fe]=0x0;/* Make Record Invalid */
      ff=IDEDisk(READ_DISK,CurrDrive,0,0,1,1,MBR,p2DriveParam);
      if(ff||(*(unsigned short*)&MBR[0x1fe]!=0xAA55)) continue;/* Success ? */
      for (i=0,CurrMBROff=0x1be;i<4;i++,CurrMBROff+=16){/*For all prim. part*/
  if(Valid(&MBR[CurrMBROff])){/* Is Current Partition Item Valid ? */
    AllPart[*CurrPart].Drive=CurrDrive;/* Set drive */
    memcpy(&AllPart[(*CurrPart)++],&MBR[CurrMBROff],16);
    if(MBR[CurrMBROff+0x4]==0x5){/* Is it extended partition ? */
      Temp[0x1fe]=0x0;FHead=AllPart[(*CurrPart)-1].FirstHead;
      FSectorTrack=AllPart[(*CurrPart)-1].FirstSectorTrack;
      ff=IDEDisk(READ_DISK,CurrDrive,FHead,CalcTrack(FSectorTrack),
      FSectorTrack&0x3f,1,Temp,p2DriveParam);/* Read Ext */
      if(ff||(*(unsigned short*)&Temp[0x1fe]!=0xAA55)){/* Success ? */
        printf("Invalid Extended Record !\n");return -1;}
      do{
        AllPart[*CurrPart].Drive=CurrDrive;/* Set drive */
        memcpy(&AllPart[(*CurrPart)++],&Temp[0x1be],16);
        if(Valid(&Temp[0x1ce])){/* Is Extended partition item valid ? */
    Temp[0x1fe]=0x0;FHead=Temp[0x1ce+1];/* Make buffer invalid */
    FSectorTrack=*(unsigned short*)&Temp[0x1ce+2];
    ff=IDEDisk(READ_DISK,CurrDrive,FHead,CalcTrack(FSectorTrack)
          ,FSectorTrack&0x3f,1,Temp,p2DriveParam);/*Read n*/
    if(ff||(*(unsigned short*)&Temp[0x1fe]!=0xAA55)){/*Success ?*/
      printf("Invalid Extended Record !\n");return -1;}
        }else break;
      }while (1);
    }
  } else break;
      }
    }
 return 0;/* Let's go home ! */
}

/****************************************************************************/
/*            Find all partitions that are supported by TUOX        */
/****************************************************************************/
void DetectTuoxPart(int *TuoxPartCnt,PartInfo *AllPart,int PartCnt,
        int *TuoxPart){
  int i;
    for(i=0,*TuoxPartCnt=0;i<PartCnt;i++)
      if(AllPart[i].OSId==TUOX_ID) TuoxPart[(*TuoxPartCnt)++]=i;
}
/****************************************************************************/
/*    Check if blockno is valid and calculate physical CHS address      */
/****************************************************************************/
int CheckAndCalc(PartInfo *p2Part,TypeDriveParam *p2DriveParam,
     unsigned char *drive,unsigned char *head,
     unsigned short *track,unsigned char *sector,
     unsigned long blockno ){
  unsigned long FirstSector=blockno*IDEDRV_BLOCK_SIZE,MaxHeadSect,FSmodMaxHS;
    if((FirstSector+IDEDRV_BLOCK_SIZE)>p2Part->TotalSectors)
      return (int)IDEDRV_BLK_NO;
    *drive=p2Part->Drive;
    MaxHeadSect=(unsigned long)(p2DriveParam[*drive-0x80].MaxHead+1)
    *p2DriveParam[*drive-0x80].MaxSect;
    FirstSector+=CalcTrack(p2Part->FirstSectorTrack)*MaxHeadSect
    +p2Part->FirstHead*(unsigned long)p2DriveParam[*drive-0x80].
    MaxSect+(p2Part->FirstSectorTrack&0x3f)-1UL;
    *track=FirstSector/MaxHeadSect;
    FSmodMaxHS=FirstSector%MaxHeadSect;
    *head=(FSmodMaxHS)/p2DriveParam[*drive-0x80].MaxSect;
    *sector=(FSmodMaxHS)%p2DriveParam[*drive-0x80].MaxSect+1;
 return 0;
}
/****************************************************************************/
/*          Displays all detected partitions        */
/****************************************************************************/
#ifdef SHOW_PARTITION_DETAIL
void PrintPartition(PartInfo *Pt,int N){
 int i,Pn=1,Drive;
   Drive=Pt[0].Drive;
   printf("\n1 Block = 1024 Bytes\n");
   printf("Device\tBoot\tStart\tEnd\tBlocks\tId\tSystem\n");
   for(i=0;i<N;i++){/* For all detected partitions */
     if(Drive!=Pt[i].Drive){Drive=Pt[i].Drive;Pn=1;
       printf("\n");}/* Reset Part. counter */
     printf("hd%c%d\t%c\t%d\t%d\t%lu\t%x\t%s\n",Pt[i].Drive-0x80+'a',
       Pn++,Pt[i].Active?'*':' ',CalcTrack(Pt[i].FirstSectorTrack),
       CalcTrack(Pt[i].LastSectorTrack),Pt[i].TotalSectors/2,
       Pt[i].OSId,IDtoText(Pt[i].OSId));/* Print partition info */
   }
}
#endif
/****************************************************************************/
/*          Show Drive Info           */
/****************************************************************************/
#ifdef SHOW_DRIVE_INFO
void PrintDriveInfo(TypeDriveParam *AllDrives){
  int i;
    for(i=0;i<4;i++){/* Display All Detected Drives */
      if(!AllDrives[i].DriveValid) break;
      printf("Drive %d: %s, ",AllDrives[i].Hddx,AllDrives[i].Model);
      printf("Mode: %s, Cyl: %d, Heads: %d, Sectors: %d\n",
       AllDrives[i].LBAEnabled?"LBA":"CHS",AllDrives[i].MaxCyl+1,
       AllDrives[i].MaxHead+1,AllDrives[i].MaxSect);
    }
}
#endif

/****************************************************************************/
/*          Initialize IDE Driver           */
/****************************************************************************/
int IDEdrv_Init(void){
  IdentifyDrives(AllDrives);/* Identify All Available Drives */
  ScanDrives(&PartCnt,AllPart,AllDrives);/* Detect All Available Partitions*/
  DetectTuoxPart(&TuoxPartCnt,AllPart,PartCnt,TuoxPart);/* Find TUOX Parts */
  DrvInit=1;
  return TuoxPartCnt;/* Return number of TUOX partitions */
}

/****************************************************************************/
/*          IDE Driver Commands           */
/****************************************************************************/
unsigned long IDEdrv_IO(int cmd,unsigned part,unsigned long blockno,
      void *p2Buff){
  unsigned char drive,head,sector;
  unsigned short track;
    if(!DrvInit) return IDEDRV_NOTINIT;
    if(cmd==IDEDRV_IDN_PART) return TuoxPartCnt;
    if(part>=TuoxPartCnt) return IDEDRV_PART_NO;
    switch (cmd){
      case IDEDRV_READ:
  if(CheckAndCalc(&AllPart[TuoxPart[part]],AllDrives,&drive,&head,
      &track,&sector,blockno)) return IDEDRV_BLK_NO;
  if(IDEDisk(READ_DISK,drive,head,track,sector,IDEDRV_BLOCK_SIZE,
       p2Buff,AllDrives)) return IDEDRV_READ_ERR;
      break;
      case IDEDRV_WRITE:
  if(CheckAndCalc(&AllPart[TuoxPart[part]],AllDrives,&drive,&head,
      &track,&sector,blockno)) return IDEDRV_BLK_NO;
  if(IDEDisk(WRITE_DISK,drive,head,track,sector,IDEDRV_BLOCK_SIZE,
       p2Buff,AllDrives)) return IDEDRV_WRITE_ERR;
      break;
      case IDEDRV_IDN_BLK:
  return AllPart[TuoxPart[part]].TotalSectors/IDEDRV_BLOCK_SIZE;
      default:return IDEDRV_UNKNOWN_CMD;
  }
  return IDEDRV_SUCCESS;
}
/****************************************************************************/
/*          Test Code             */
/****************************************************************************/
#ifdef TEST_CODE_ALLOWED
int main (void){
  IDEdrv_Init();/* Init IDE Driver */
#ifdef SHOW_COPYRIGHT
  printf("IDE low level layer for TUOX OS v1.1, (c)1997-98 Petr Pomkla & Karel Miarka\n");
  printf("Technical University Ostrava, Jan 21 1998, %d TUOX partition(s) found.\n",
   TuoxPartCnt);
#endif
#ifdef SHOW_DRIVE_INFO
  PrintDriveInfo(AllDrives);/* Print Drive Info */
#endif
#ifdef SHOW_PARTITION_DETAIL
  PrintPartition(AllPart,PartCnt);/* Display All Partitions */
  printf("\n");
#endif
  return 0;
 }
#endif
/****************************************************************************/
/*          IDE Task              					    */
/****************************************************************************/
void IDE_task(void){
  static message msg,CpyMsg;
  char Data[1024];
  int r,proc_nr;
  IDEdrv_Init(); /* Init driver */
  while(TRUE){/* Forever loop */
    receive(ANY,msg);//Recv Mess
    proc_nr=msg.m_source;
    switch (msg.m_type){
      case DEV_READ://Read or Write Command
	if(IDEdrv_IO(IDEDRV_READ,msg.m_u.m_dio.device,
	   msg.m_u.m_dio.position,Data)==IDEDRV_SUCCESS) {
	   msg.REP_STATUS=1;
	   /* PUT DATA */
	   CpyMsg.m_source=HDD;
	   CpyMsg.m_type=SYS_COPY;
	   /* SOURCE */
	   CpyMsg.SRC_SEG=(char) D;
	   CpyMsg.SRC_PROC=(int) HDD;
	   CpyMsg.SRC_VIR=(long) Data;
	   /* destination */
	   CpyMsg.DST_SEG=(char) D;
	   CpyMsg.DST_PROC=msg.m_u.m_dio.proc_nr;
	   CpyMsg.DST_VIR=msg.m_u.m_dio.address;
	   CpyMsg.BYTE_CNT=1024;
	   send(SYSTASK,&CpyMsg);
	}else msg.REP_STATUS=EIO;
	break;
      case DEV_WRITE:
	/* GET DATA */
	CpyMsg.m_source=HDD;
	CpyMsg.m_type=SYS_COPY;
	/* SOURCE */
	CpyMsg.SRC_SEG=(char) D;
	CpyMsg.SRC_PROC=(int) msg.m_u.m_dio.proc_nr;
	CpyMsg.SRC_VIR=(long) msg.m_u.m_dio.address;
	/* destination */
	CpyMsg.DST_SEG=(char) D;
	CpyMsg.DST_PROC=HDD;
	CpyMsg.DST_VIR=(long) Data;
	CpyMsg.BYTE_CNT=1024;
	send(SYSTASK,&CpyMsg);
	if(IDEdrv_IO(IDEDRV_WRITE,msg.m_u.m_dio.device,
	   msg.m_u.m_dio.position,Data)==IDEDRV_SUCCESS) msg.REP_STATUS=1;
	else msg.REP_STATUS=EIO;
	break;
      case DEV_IOCTL:
	switch(msg.m_u.m_dio.count) {
	  case GETDEVSIZE: /* no. of blocks */
	  { long l;
	    l=IDEdrv_IO(IDEDRV_IDN_BLK,(unsigned)msg.m_u.m_dio.device,0,0);
	    if((l!=IDEDRV_PART_NO)&&(l!=IDEDRV_NOTINIT)){
	      msg.REP_STATUS=1;
	      /* return long */
	      CpyMsg.m_source=HDD;
	      CpyMsg.m_type=SYS_COPY;
	      /* SOURCE */
	      CpyMsg.SRC_SEG=(char) D;
	      CpyMsg.SRC_PROC=(int) HDD;
	      CpyMsg.SRC_VIR=(long) &l;
	      /* destination */
	      CpyMsg.DST_SEG=(char) D;
	      CpyMsg.SRC_PROC=(int) msg.m_u.m_dio.proc_nr;
	      CpyMsg.SRC_VIR=(long) msg.m_u.m_dio.address;
	      CpyMsg.BYTE_CNT=sizeof(long);
	      send(SYSTASK,&CpyMsg);
	    } else msg.REP_STATUS=EIO;
	  }
	  break;
	  case GETNUMBERMINORDEV:  /* No. of minor devices */
	  { int i;
	   if((i=IDEdrv_IO(IDEDRV_IDN_PART,0,0,0))!=IDEDRV_NOTINIT){
	      msg.REP_STATUS=1;
	      /* return int */
	      CpyMsg.m_source=HDD;
	      CpyMsg.m_type=SYS_COPY;
	      /* SOURCE */
	      CpyMsg.SRC_SEG=(char) D;
	      CpyMsg.SRC_PROC=(int) HDD;
	      CpyMsg.SRC_VIR=(long) &i;
	      /* destination */
	      CpyMsg.DST_SEG=(char) D;
	      CpyMsg.DST_PROC=msg.m_u.m_dio.proc_nr;
	      CpyMsg.DST_VIR=msg.m_u.m_dio.address;
	      CpyMsg.BYTE_CNT=sizeof(int);
	      send(SYSTASK,&CpyMsg);
	    } else msg.REP_STATUS=EIO;
	  }
	  break;
	}
	break;
    }
    msg.m_type=TASK_REPLY;
    send(proc_nr,msg);
  }
}
