/****************************************************************************/
/*            IDE Low Level Support Layer for TUOX OS v1.1		    */
/*    Written by Petr Pomkla & Karel Miarka, Technical University Ostrava   */
/*               Copyright (c)1997-98 Petr Pomkla. Jan 21 1998              */
/*    			       (Header File)			            */                                                /*  Notes: CHS Addressing    Yes              */
/*     LBA Addressing    Yes              				    */
/*     PIO Mode          Yes   					            */
/****************************************************************************/
#include <DOS.H>
#include <STDIO.H>
#include <STRING.H>
#include "ioctl.h"
#include "type.h"
#include "com.h"
#include "makra.h"
#include "const.h"
#include "errno.h"
#include "syst_pub.h"

/****************************************************************************/
/*             Some Definitions           */
/****************************************************************************/

/* IDE registers */
#define DATA_REG              0x1f0
#define ERROR_REG             0x1f1
#define FEATURE_REG           0x1f1
#define SECTOR_COUNT_REG      0x1f2
#define SECTOR_NUMBER_REG     0x1f3
#define CYLINDER_LOW_REG      0x1f4
#define CYLINDER_HI_REG       0x1f5
#define DRIVE_HEAD_REG        0x1f6
#define STATUS_REG            0x3f6
#define COMMAND_REG           0x1f7
/* Error Bits */
#define ADDRES_MARK_NOT_FOUND 0x1
#define TRACK_0_NOT_FOUND     0x2
#define ABORT         	      0x4
#define SECTOR_NOT_FOUND      0x10
#define UNCORRECT_DATA        0x40
#define BAD_BLOCK_MARK        0x80
/* Status Bits */
#define ERROR         	      0x1
#define INDEX                 0x2
#define DATA_REQUEST          0x8
#define WRITE_FAULT           0x20
#define READY                 0x40
#define BUSY                  0x80
/* IDE Commands */
#define IDENTIFY_DRIVE        0xEC
#define READ_SECTORS          0x20
#define WRITE_SECTORS         0x30
/* Drives */
#define MASTER_DRIVE          0x0
#define SLAVE_DRIVE           0x10
/* Mode Bits */
#define LBA_MODE              0x40
/* High Level IDE Commands */
#define READ_DISK             0x0
#define WRITE_DISK            0x1
/* Tuox ID */
#define TUOX_ID               0x30
/****************************************************************************/
/*            IDE Driver Constants          */
/****************************************************************************/
#define IDEDRV_BLOCK_SIZE     0x2UL /* Block size (in sectors)(1k def.) */
/* IDE Driver Commands */
#define IDEDRV_READ           0x0 /* Read one block from disk */
#define IDEDRV_WRITE          0x1 /* Write one block to disk */
#define IDEDRV_IDN_BLK        0x2 /* Get number of allocable blocks */
#define IDEDRV_IDN_PART       0x3 /* Get number of TUOX partitions */
/* IDE Driver Error codes */
#define IDEDRV_SUCCESS        0x0UL   /* Command completed successfuly */
#define IDEDRV_NOTINIT        0xffffffffUL/* Driver isn't initialized */
#define IDEDRV_PART_NO        0xfffffffeUL/* Partition number is out of r. */
#define IDEDRV_BLK_NO         0xfffffffdUL/* Block number is out of range */
#define IDEDRV_READ_ERR       0xfffffffcUL/* Read Error */
#define IDEDRV_WRITE_ERR      0xfffffffbUL/* Write Error */
#define IDEDRV_UNKNOWN_CMD    0xfffffffaUL/* Unknown command */
/****************************************************************************/
/*             Drive ID Structure         */
/****************************************************************************/
typedef struct{
  short DriveValid;   /* This record is valid */
  short LBAEnabled;   /* LBA mode is enabled */
  short Hddx;     /* Physical Drive ID 0,1,2,3 */
  unsigned short MaxHead; /* Max Head Value */
  unsigned int MaxCyl;    /* Max Cylinder Value */
  unsigned short MaxSect; /* Max Sector Value */
  char Model[41];   /* Drive Model (string) */
 }TypeDriveParam;
/****************************************************************************/
/*        Partition Item Structure        */
/****************************************************************************/
typedef struct{
  unsigned char Active;      /* Partition is Active */
  unsigned char FirstHead;     /* First head */
  unsigned short FirstSectorTrack; /* First Sector and Track */
  unsigned char OSId;      /* Operating system ID */
  unsigned char LastHead;    /* Last Head */
  unsigned short LastSectorTrack;  /* Last sector and track */
  unsigned long RelativeMove;      /* Offset from begining */
  unsigned long TotalSectors;      /* Number of sectors */
  unsigned char Drive;       /* hda = 0x80, hdb = 0x81 ... */
} PartInfo;

/****************************************************************************/
/*        			IDE Task 				    */
/****************************************************************************/
void IDE_task(void);
/* End of file */