/* CONST.H */
/* This file contains common definitions used in various parts of the system */

#ifndef __const_h
#define __const_h

#define PRIVATE       static    /* PRIVATE x limits the scope of x to the current file */
#define PUBLIC                  /* PUBLIC is the opposite of PRIVATE */

#define TRUE               1    /* used for turning integers into Booleans */
#define FALSE              0    /* used for turning integers into Booleans */
#define OK		   0    /* used as a result code indicating sucecess */ 

#define HZ                60    /* clock freq (software settable on IBM-PC) */
#define BLOCK_SIZE      1024    /* # bytes in a disk block */
#define SUPER_USER  ((uid_t) 0)    /* uid of superuser */


#ifndef NULL
 #define NULL     ((void *)0)    /* null pointer */
#endif

#define NR_SEGS            3    /* # segments per process */
#define T                  0    /* proc[i].mem_map[T] is for text */
#define D                  1    /* proc[i].mem_map[D] is for data */
#define S                  2    /* proc[i].mem_map[S] is for stack */


/* Process numbers of some important processes. */

#define NR_TASKS            7 /* how many tasks there exist (incl. idle) */
#define NR_PROCS	  100 /* maximal # of processes */
#define NR_SERVS	    2 /* # of servers */
#define MM        (NR_TASKS+NR_SERVS)    /* process number of memory manager */
#define FS        (NR_TASKS+NR_SERVS+1)    /* process number of file system */
#define INIT      (NR_TASKS+NR_SERVS+2)    /* init -- the process that goes multiuser */
#define LOW_USER  (NR_TASKS+NR_SERVS+3)    /* first user process */
 
#define END_TASK_ADDR (&proc[NR_TASKS-1])
#define BEG_USER_ADDR (&proc[LOW_USER]) /* first user process's addr */

/* Makra pro identifikaci procesu */
#define NIL_PROC          ((struct proc *) 0)
#define proc_addr(n)      (&proc[n])
#define isokprocn(n)      ((unsigned) (n) < (NR_PROCS+NR_TASKS+NR_SERVS))
#define isoksrc_dest(n)   (isokprocn(n) || (n) == ANY)
#define istaskp(p)        (((p) <= END_TASK_ADDR) && ((p) != proc_addr(IDLE)))
#define isservp(p)        (((p) < BEG_USER_ADDR-1) && ((p) > END_TASK_ADDR))
#define isuserp(p)        ( ((p) >= BEG_USER_ADDR) && ((p) <= proc_addr(NR_PROCS+NR_TASKS+NR_SERVS))  )
#define issysentn(n)      (((n) == FS) || ((n) == MM))


#define ROOT_DEV ((dev_t)  (256*FLOPPY)) /* major-minor dev # of root device - floppy */      

/* Memory is allocated in clicks (paragraphs). */
#define CLICK_SIZE        16    /* unit in which memory is allocated */
#define CLICK_SHIFT        4    /* log2 of CLICK_SIZE */

/* Flag bits for i_mode in the inode. */
#define I_TYPE          0077000 /* this field gives inode type */
#define I_REGULAR       0040000 /* regular file, not dir or special */
#define I_DIRECTORY     0020000 /* file is a directory */
#define I_BLOCK_SPECIAL 0010000 /* block special file */
#define I_CHAR_SPECIAL  0004000 /* character special file */
#define I_SET_UID_BIT   0002000 /* set effective uid_t on exec */
#define I_SET_GID_BIT   0001000 /* set effective gid_t on exec */
#define RWX_MODES       0000777 /* mode bits for RWX only */
#define OR_BIT           0000004 /* Rwx protection bit - other */
#define OW_BIT           0000002 /* rWx protection bit */
#define OX_BIT           0000001 /* rwX protection bit */
#define GR_BIT           0000040 /* Rwx protection bit - group */
#define GW_BIT           0000020 /* rWx protection bit */
#define GX_BIT           0000010 /* rwX protection bit */
#define UR_BIT           0000400 /* Rwx protection bit - user */
#define UW_BIT           0000200 /* rWx protection bit */
#define UX_BIT           0000100 /* rwX protection bit */


/* Some limits. */
#define MAX_BLOCK_NR  ((block_t) 077777777)     /* largest block number */
#define MAX_FILE_POS ((off_t) 037777777777)     /* largest legal file offset */

#define NO_BLOCK              ((block_t) 0)     /* absence of a block number */
#define NO_ENTRY              ((inode_t) 0)     /* absence of a dir entry */
#define NO_ZONE                ((zone_t) 0)     /* absence of a zone number */
#define NO_DEV                  ((dev_t) 0)     /* absence of a device numb */

#define PATH_MAX	128 /* maximal length of file path */

#endif /* #ifndef __const_h */
