/* This file comprises type definitions of types used internally by the system */
#ifndef __type_h
#define __type_h

#define TRUE 1
#define FALSE 0

typedef unsigned char Byte;
typedef unsigned int Word;
typedef unsigned long DWord;

/* types to specify number of bytes */
typedef Word  vir_bytes;
typedef DWord phys_bytes;

/* types to specify number of paragraphs (called 'clicks' here) */
typedef Word  vir_clicks;
typedef Word  phys_clicks;

typedef int uid_t;
typedef int gid_t;
typedef int pid_t;

typedef unsigned int procnr_t;

typedef unsigned int block_t;
#define zone_t block_t 
/* zones and blocks are assumed equivalent for now */

typedef unsigned long off_t; /* file position */
typedef unsigned int mode_t;
typedef unsigned int inode_t;
typedef unsigned int dev_t;

typedef unsigned int sigset_t;
 
#define d_minor(d) (d & 0xFF)
#define d_major(d) ((d & 0xFF00) >> 8)

typedef unsigned char link_t;
typedef long real_time; /* in seconds since Jan 1, 1970 */

typedef void (*sighandler_t) (int); 


typedef unsigned int reg_t; /* type to represent processor register's value */

/* Types relating to messages. */
#define M1                 1
#define M3                 3
#define M4                 4
#define M3_STRING         14

typedef struct {int m1i1, m1i2, m1i3; char *m1p1, *m1p2, *m1p3;} mess_1;
typedef struct {int m2i1, m2i2, m2i3; long m2l1, m2l2; char *m2p1;} mess_2;
typedef struct {int m3i1, m3i2; char *m3p1; char m3ca1[M3_STRING];} mess_3;
typedef struct {long m4l1, m4l2, m4l3, m4l4, m4l5;} mess_4;
typedef struct {char m5c1, m5c2; int m5i1, m5i2; long m5l1, m5l2, m5l3;}mess_5;
typedef struct {int m6i1, m6i2, m6i3; long m6l1; sighandler_t m6f1;} mess_6;
typedef struct {
 int device;          /* minor dev nr */ 
 int proc_nr;         /* process requesting I/O */ 
 unsigned int count;  /* # of bytes required */ 
 unsigned long position;  /* starting position */
                          /* (TTY uses this for flags if needed) */
 vir_bytes address;   /* data buffer address in (proc_nr) space */
                      /* (TTY uses it as ptr to info structure for ioctl */		  
} mess_devio;
                                        
#define DEV_READ  100
#define DEV_WRITE 101
#define DEV_IOCTL 102                                        
#define DEV_OPEN  103
#define DEV_CLOSE 104

typedef struct {
  int m_source;                 /* who sent the message */
  int m_type;                   /* what kind of message is it */
  union {
        mess_1 m_m1;
        mess_2 m_m2;
        mess_3 m_m3;
        mess_4 m_m4;
        mess_5 m_m5;
        mess_6 m_m6;
        mess_devio m_dio;
  } m_u;
} message;

/* The following defines provide names for useful members. */
#define m1_i1  m_u.m_m1.m1i1
#define m1_i2  m_u.m_m1.m1i2
#define m1_i3  m_u.m_m1.m1i3
#define m1_p1  m_u.m_m1.m1p1
#define m1_p2  m_u.m_m1.m1p2
#define m1_p3  m_u.m_m1.m1p3

#define m2_i1  m_u.m_m2.m2i1
#define m2_i2  m_u.m_m2.m2i2
#define m2_i3  m_u.m_m2.m2i3
#define m2_l1  m_u.m_m2.m2l1
#define m2_l2  m_u.m_m2.m2l2
#define m2_p1  m_u.m_m2.m2p1

#define m3_i1  m_u.m_m3.m3i1
#define m3_i2  m_u.m_m3.m3i2
#define m3_p1  m_u.m_m3.m3p1
#define m3_ca1 m_u.m_m3.m3ca1

#define m4_l1  m_u.m_m4.m4l1
#define m4_l2  m_u.m_m4.m4l2
#define m4_l3  m_u.m_m4.m4l3
#define m4_l4  m_u.m_m4.m4l4
#define m4_l5  m_u.m_m4.m4l5

#define m5_c1  m_u.m_m5.m5c1
#define m5_c2  m_u.m_m5.m5c2
#define m5_i1  m_u.m_m5.m5i1
#define m5_i2  m_u.m_m5.m5i2
#define m5_l1  m_u.m_m5.m5l1
#define m5_l2  m_u.m_m5.m5l2
#define m5_l3  m_u.m_m5.m5l3

#define m6_i1  m_u.m_m6.m6i1
#define m6_i2  m_u.m_m6.m6i2
#define m6_i3  m_u.m_m6.m6i3
#define m6_l1  m_u.m_m6.m6l1
#define m6_f1  m_u.m_m6.m6f1


#define TASK_REPLY 32700
#define REP_PROC_NR m1_i2
#define REP_STATUS m1_i1
#define OK 0

struct mem_map {
  vir_clicks mem_vir;           /* virtual address */
  phys_clicks mem_phys;         /* physical address */
  vir_clicks mem_len;           /* length */
};

#endif

